@@ -18,7 +18,7 @@ use RPi::Serial;
 use RPi::SPI;
 use RPi::WiringPi::Constant qw(:all);
 
-our $VERSION = '2.3618';
+our $VERSION = '2.3619';
 
 my $fatal_exit = 1;
 
@@ -35,7 +35,6 @@ BEGIN {
     $SIG{__DIE__} = \&_error;
     $SIG{INT} = \&_error;
 };
-
 # core
 
 sub new {
@@ -91,21 +90,26 @@ sub adc {
     my $adc;
 
     if (defined $args{model} && $args{model} eq 'MCP3008'){
-        $adc = RPi::ADC::MCP3008->new($args{channel});
+        my $pin = $self->pin($args{channel});
+        $adc = RPi::ADC::MCP3008->new($pin->num);
     }
     else {
+        # ADSxxxx ADCs don't require any pins
         $adc = RPi::ADC::ADS->new(%args);
     }
     return $adc;
 }
 sub dac {
     my ($self, %args) = @_;
+    my $cs_pin = $self->pin($args{cs});
+    my $shdn_pin = $self->pin($args{shdn}) if defined $args{shdn};
     $args{model} = 'MCP4922' if ! defined $args{model};
     my $dac = RPi::DAC::MCP4922->new(%args);
     return $dac;
 }
 sub dpot {
     my ($self, $cs, $channel) = @_;
+    $self->register($self->pin($cs));
     my $dpot = RPi::DigiPot::MCP4XXXX->new($cs, $channel);
     return $dpot;
 }
@@ -120,7 +124,7 @@ sub pin {
     my $pins_in_use = $self->registered_pins;
     my $gpio = $self->pin_to_gpio($pin_num);
 
-    if (defined $ENV{RPI_PINS} && grep {$gpio == $_} split /,/, $pins_in_use){
+    if (grep {$gpio == $_} @{ $self->registered_pins }){
         die "\npin $pin_num is already in use... can't create second object\n";
     }
 
@@ -137,8 +141,21 @@ sub i2c {
     return RPi::I2C->new($addr, $i2c_device);
 }
 sub lcd {
-    my $self = shift;
+    my ($self, %args) = @_;
+
+    # pre-register all pins so we can clean them up
+    # accordingly upon cleanup
+
+    for (qw(rs strb d0 d1 d2 d3 d4 d5 d6 d7)){
+        if (! exists $args{$_} || $args{$_} !~ /^\d+$/){
+            die "lcd() requires pin configuration within a hash\n";
+        }
+        next if $args{$_} == 0;
+        my $pin = $self->pin($args{$_});
+    }
+
     my $lcd = RPi::LCD->new;
+    $lcd->init(%args);
     return $lcd;
 }
 sub bmp {
@@ -147,6 +164,7 @@ sub bmp {
 }
 sub hygrometer {
     my ($self, $pin) = @_;
+    $self->register_pin($pin);
     my $sensor = RPi::DHT11->new($pin);
     return $sensor;
 }
@@ -157,7 +175,14 @@ sub spi {
 }
 sub shift_register {
     my ($self, $base, $num_pins, $data, $clk, $latch) = @_;
-    $self->shift_reg_setup($base, $num_pins, $data, $clk, $latch);
+
+    my @pin_nums;
+
+    for ($data, $clk, $latch){
+        my $pin = $self->pin($_);
+        push @pin_nums, $pin->num;
+    }
+    $self->shift_reg_setup($base, $num_pins, @pin_nums);
 }
 
 # private
@@ -347,7 +372,7 @@ various items
     my $temp      = $env->temp; # celcius
     my $farenheit = $env->temp('f');
 
-    # GPS (requires gpsd to be installed and running
+    # GPS (requires gpsd to be installed and running)
 
     my $gps = $pi->gps;
 
@@ -360,9 +385,7 @@ various items
     # LCD
     #
 
-    my $lcd = $pi->lcd;
-
-    $lcd->init(...);
+    my $lcd = $pi->lcd(...);
 
     # first column, first row
     
@@ -385,7 +408,7 @@ This is the root module for the C<RPi::WiringPi> system. It interfaces to a
 Raspberry Pi board, its accessories and its GPIO pins via the
 L<wiringPi|http://wiringpi.com> library through the Perl wrapper
 L<WiringPi::API|https://metacpan.org/pod/WiringPi::API>
-module.
+module, and various other custom device specific  modules.
 
 L<wiringPi|http://wiringpi.com> must be installed prior to installing/using
 this module (v2.36+).
@@ -414,7 +437,7 @@ command.
 See L<RPi::WiringPi::Util> for utility/helper methods that are imported into
 an C<RPi::WiringPi> object.
 
-=head2 new(%args)
+=head2 new([%args])
 
 Returns a new C<RPi::WiringPi> object. We exclusively use the C<GPIO>
 (Broadcom (BCM) GPIO) pin numbering scheme. These pin numbers are printed on the
@@ -441,11 +464,14 @@ Parameters:
 
 Mandatory, Integer: The pin number to attach to.
 
-=head2 lcd()
+=head2 lcd(...)
 
 Returns a L<RPi::LCD> object, which allows you to fully manipulate
 LCD displays connected to your Raspberry Pi.
 
+Please see the linked documentation for information regarding the parameters
+required.
+
 =head2 i2c($addr, [$device])
 
 Creates a new L<RPi::I2C> device object which allows you to communicate with
